#include <eGizmo_SerialLCD.h>
#include <Arduino.h>
#include <SoftwareSerial.h>

/* this is just here for reference
#define INPUT             0x00
#define INPUT_PULLUP      0x02

#define HIGH 0x1
#define LOW  0x0
*/

eGizmo_SerialLCD::eGizmo_SerialLCD(SoftwareSerial *ss, int lines, int pos){

	hwSerial = NULL;
	swSerial = ss;
	mySerial = swSerial;

	lineHold = lines;
	posHold = pos;

}

eGizmo_SerialLCD::eGizmo_SerialLCD(HardwareSerial *hs, int lines, int pos){
	
	swSerial = NULL;
	hwSerial = hs;
	mySerial = hwSerial;

	lineHold = lines;
	posHold = pos;

}

bool eGizmo_SerialLCD::begin(void){
	
	if (hwSerial) hwSerial->begin(9600);

	if (swSerial) swSerial->begin(9600);

	//send command
	cmdWrite(TEST);



	if(replyReceiver() == "OK"){

		cmdWrite(INIT);
		
		if(replyReceiver() == "OK"){
		
			cmdWrite(CLEAR);
		
			if(replyReceiver() == "OK"){
				return true;
			}else{
				return false;
			}//else
		
		}else{
			return false;
		}//else

	}else{
		return false;
	}//else	
	
}

void eGizmo_SerialLCD::end(void){

	if (hwSerial) hwSerial->end();

	if (swSerial) swSerial->end();

}

bool eGizmo_SerialLCD::clear(void){
	
	cmdWrite(CLEAR);

	if(replyReceiver() == "OK"){
		return true;
	}else{
		return false;
	}//else
}

bool eGizmo_SerialLCD::scroll(int address, String message){
	//variables
	String commander = "";
	int line = constrain(address, 1, 2);

	//main

	//constrain message length to limit
	if(message.length() <= 64){

		//------ Set Line number ------//
		
		if(line == 1){
			commander = SCROLL1;
		}//if
		
		if(line == 2){
			commander = SCROLL2;
		}//if
		
		//------ Set Line number ------//
		
		//------ Set message format ------//
		
		for(int i = 0; i < 15; i++){
			commander += " ";
		}//for
		
		//Add message to command
		commander += message;
		
		//------ Set message format ------//
		
		//send command
		cmdWrite(commander);
		
		//------ Reply Parsing ------//

		if(replyReceiver() == "OK"){
			return true;
		}else{
			return false;
		}//else

		//------ Reply Parsing ------//
	}else{
		return false;
	}//else
}

bool eGizmo_SerialLCD::print(String message, int line, int pos){
	//variables
	String commander = "";
	int lineAddress = constrain(line, 1, lineHold);
	int posAddress = constrain(pos, 1, posHold);
	bool proceed = false;

	//main

	//------ Set Line position ------// 
	
	commander = CURPOS;
	commander += String(posAddress-1);

	//send command
	cmdWrite(commander);

	if(replyReceiver() != "OK"){
		return false;
	}//if

	//------ Set Line position ------// 

	//------ Set Line number ------//

	//Set command text
	switch(lineAddress){
		case 1:
			commander = WL1;
		break;
	
		case 2:
			commander = WL2;
		break;

		case 3:
			commander = WL3;
		break;

		case 4:
			commander = WL4;
		break;
	}//switch

	//add message to command
	commander += message;

	//send command to SerialLCD
	cmdWrite(commander);
	//------ Set Line number ------//

	//------ Reply Parsing ------//

	if(replyReceiver() == "OK"){
		return true;
	}else{
		return false;
	}//else

	//------ Reply Parsing ------//

}

bool eGizmo_SerialLCD::printOver(String message, int line, int pos){
	//variables
	String commander = "";
	int lineAddress = constrain(line, 1, lineHold);
	int posAddress = constrain(pos, 1, posHold);
	bool proceed = false;

	//main

	//------ Set Line number ------//

	//Set command text
	switch(lineAddress){
		case 1:
			commander = OL1; 
		break;
	
		case 2:
			commander = OL2;
		break;

		case 3:
			commander = OL3;
		break;

		case 4:
			commander = OL4;
		break;
	}//switch

	//Add Spaces
	for(int i = 0; i < (posAddress-1); i++){
		commander += " ";
	}//for

	//add message to command
	commander += message;

	//send command to SerialLCD
	cmdWrite(commander);
	//------ Set Line number ------//

	//------ Reply Parsing ------//

	if(replyReceiver() == "OK"){
		return true;
	}else{
		return false;
	}//else

	//------ Reply Parsing ------//

}

bool eGizmo_SerialLCD::pinMode(int pinNum, uint8_t modal){
	//variables
	String commander = "";
	bool pullup = false;
	//main

	//constrain pinNum to valid values
	if((pinNum >= 0) && (pinNum <= 10)){

		//------ Set Mode Command ------//

		if(modal == INPUT){

			commander = SETIN;
		
		}else if(modal == INPUT_PULLUP){
		
			commander = SETIN;
			pullup = true;
		
		}else if(modal == OUTPUT){
		
			commander = SETOUT;

		}

		//------ Set Mode Command ------//

		//Add pin number to command
		commander += String(pinNum);
		
		//send command
		cmdWrite(commander);

		//------ Reply Parsing ------//

		if(replyReceiver() == "OK"){
			//if pullup
			if(pullup){
				//------ Pull up routine------//

				commander = WRITEHIGH;
				commander += String(pinNum);

				//send command
				cmdWrite(commander);

				//------ Pull up routine ------//

				//------ Reply Parsing ------//

				if(replyReceiver() == "OK"){
					return true;
				}else{
					return false;
				}//else

				//------ Reply Parsing ------//
			
			}else{
				return true;
			}//else

		}else{
			return false;
		}//else

		//------ Reply Parsing ------//

	}else{
		return false;
	}//else
}

bool eGizmo_SerialLCD::digitalWrite(int pinNum, uint8_t val){
	//variables
	String commander = "";
	
	//main

	//constrain pinNum to valid values
	if((pinNum >= 0) && (pinNum <= 10)){

		//------ Set command ------//
		
		if(val == HIGH){
			commander = WRITEHIGH;
		}else if(val == LOW){
			commander = WRITELOW;
		}//else if

		//------ Set command ------//

		//add pin number to command
		commander += String(pinNum);

		//send command
		cmdWrite(commander);

		//------ Reply Parsing ------//

		if(replyReceiver() == "OK"){
			return true;
		}else{
			return false;
		}//else

		//------ Reply Parsing ------//
	
	}else{
		return false;
	}

}

uint8_t eGizmo_SerialLCD::digitalRead(int pinNum){
	//varibles
	String commander = "";

	//main
	//constrain pinNum to valid values
	if((pinNum >= 0) && (pinNum <= 10)){

		//------ Reply Parsing ------//
		String response = replyReceiver();

		if( response != "ERR"){
			if(response == "0"){
				return LOW;
			}else if(response == "1"){
				return HIGH;
			}else{
				return -1;
			}
		}else{
			return -1;
		}//else
		//------ Reply Parsing ------//

	}else{
		return -1;
	}
}


void eGizmo_SerialLCD::cmdWrite(String cmd){

	mySerial->write(STX);
	mySerial->print(cmd);
	mySerial->write(ETX);

}

String eGizmo_SerialLCD::replyReceiver(int timer){
	String receiver= "";
	bool acceptor = false;
	char inChar;
	
	while(inChar != 0x03){
		inChar = mySerial->read();
		if(inChar > 0 ){
			if(inChar == 0x03){
				acceptor = false;
				return receiver;
			}
		
			if((acceptor) && (String(inChar) != "")){
				receiver += inChar;
			}
		
			if(inChar == 0x02){
				acceptor = true;
			}
		}
	}
}